@@ -42,6 +42,7 @@
 #  include "config.h"
 #endif
 
+#include "gstalsaelements.h"
 #include "gstalsamidisrc.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_alsa_midi_src_debug);
@@ -300,11 +301,12 @@ enum
   PROP_LAST,
 };
 
-#define _do_init \
-    GST_DEBUG_CATEGORY_INIT (gst_alsa_midi_src_debug, "alsamidisrc", 0, "alsamidisrc element");
 #define gst_alsa_midi_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstAlsaMidiSrc, gst_alsa_midi_src, GST_TYPE_PUSH_SRC,
-    _do_init);
+    GST_DEBUG_CATEGORY_INIT (gst_alsa_midi_src_debug, "alsamidisrc", 0,
+        "alsamidisrc element"));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (alsamidisrc, "alsamidisrc",
+    GST_RANK_PRIMARY, GST_TYPE_ALSA_MIDI_SRC, alsa_element_init (plugin));
 
 static void gst_alsa_midi_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -418,7 +420,7 @@ push_buffer (GstAlsaMidiSrc * alsamidisrc, gpointer data, guint size,
   GST_BUFFER_DTS (buffer) = time;
   GST_BUFFER_PTS (buffer) = time;
 
-  local_data = g_memdup (data, size);
+  local_data = g_memdup2 (data, size);
 
   gst_buffer_append_memory (buffer,
       gst_memory_new_wrapped (0, local_data, size, 0, size, local_data,
