@@ -25,6 +25,7 @@
 
 #include <gst/gst.h>
 
+#include "gstglelements.h"
 #include "gstglmixerbin.h"
 
 #define GST_CAT_DEFAULT gst_gl_mixer_bin_debug
@@ -41,6 +42,7 @@ typedef enum
   GST_GL_MIXER_BIN_START_TIME_SELECTION_SET
 } GstGLMixerBinStartTimeSelection;
 
+#define GST_TYPE_GL_MIXER_BIN_START_TIME_SELECTION (gst_gl_mixer_bin_start_time_selection_get_type())
 static GType
 gst_gl_mixer_bin_start_time_selection_get_type (void)
 {
@@ -121,6 +123,7 @@ enum
   PROP_LATENCY,
   PROP_START_TIME_SELECTION,
   PROP_START_TIME,
+  PROP_CONTEXT,
 };
 
 enum
@@ -137,6 +140,8 @@ G_DEFINE_TYPE_WITH_CODE (GstGLMixerBin, gst_gl_mixer_bin, GST_TYPE_BIN,
     G_ADD_PRIVATE (GstGLMixerBin)
     G_IMPLEMENT_INTERFACE (GST_TYPE_CHILD_PROXY,
         gst_gl_mixer_bin_child_proxy_init));
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glmixerbin, "glmixerbin",
+    GST_RANK_NONE, GST_TYPE_GL_MIXER_BIN, gl_element_init (plugin));
 
 static guint gst_gl_mixer_bin_signals[LAST_SIGNAL] = { 0 };
 
@@ -194,7 +199,7 @@ gst_gl_mixer_bin_class_init (GstGLMixerBinClass * klass)
   g_object_class_install_property (gobject_class, PROP_START_TIME_SELECTION,
       g_param_spec_enum ("start-time-selection", "Start Time Selection",
           "Decides which start time is output",
-          gst_gl_mixer_bin_start_time_selection_get_type (),
+          GST_TYPE_GL_MIXER_BIN_START_TIME_SELECTION,
           DEFAULT_START_TIME_SELECTION,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -204,6 +209,12 @@ gst_gl_mixer_bin_class_init (GstGLMixerBinClass * klass)
           G_MAXUINT64,
           DEFAULT_START_TIME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_CONTEXT,
+      g_param_spec_object ("context",
+          "OpenGL context",
+          "Get OpenGL context",
+          GST_TYPE_GL_CONTEXT, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstMixerBin::create-element:
    * @object: the #GstGLMixerBin
@@ -214,8 +225,7 @@ gst_gl_mixer_bin_class_init (GstGLMixerBinClass * klass)
    */
   gst_gl_mixer_bin_signals[SIGNAL_CREATE_ELEMENT] =
       g_signal_new ("create-element", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_generic,
-      GST_TYPE_ELEMENT, 0);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, GST_TYPE_ELEMENT, 0);
 
   gst_element_class_add_static_pad_template (element_class, &src_factory);
 
@@ -228,6 +238,8 @@ gst_gl_mixer_bin_class_init (GstGLMixerBinClass * klass)
   gst_element_class_set_metadata (element_class, "OpenGL video_mixer empty bin",
       "Bin/Filter/Effect/Video/Mixer", "OpenGL video_mixer empty bin",
       "Matthew Waters <matthew@centricular.com>");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_GL_MIXER_BIN_START_TIME_SELECTION, 0);
 }
 
 static void
