@@ -45,6 +45,7 @@
 #include "config.h"
 #endif
 
+#include "gstglelements.h"
 #include "gstgltransformation.h"
 
 #include <gst/gl/gstglapi.h>
@@ -89,6 +90,8 @@ enum
 
 G_DEFINE_TYPE_WITH_CODE (GstGLTransformation, gst_gl_transformation,
     GST_TYPE_GL_FILTER, DEBUG_INIT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (gltransformation, "gltransformation",
+    GST_RANK_NONE, GST_TYPE_GL_TRANSFORMATION, gl_element_init (plugin));
 
 static void gst_gl_transformation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -762,10 +765,22 @@ gst_gl_transformation_gl_start (GstGLBaseFilter * base_filter)
     return FALSE;
 
   if (gst_gl_context_get_gl_api (base_filter->context)) {
+    gchar *frag_str;
+    gboolean ret;
+
+    frag_str =
+        gst_gl_shader_string_fragment_get_default (base_filter->context,
+        GST_GLSL_VERSION_NONE,
+        GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
+
     /* blocking call, wait until the opengl thread has compiled the shader */
-    return gst_gl_context_gen_shader (base_filter->context,
+    ret = gst_gl_context_gen_shader (base_filter->context,
         gst_gl_shader_string_vertex_mat4_vertex_transform,
-        gst_gl_shader_string_fragment_default, &transformation->shader);
+        frag_str, &transformation->shader);
+
+    g_free (frag_str);
+
+    return ret;
   }
   return TRUE;
 }
@@ -792,7 +807,7 @@ gst_gl_transformation_prepare_output_buffer (GstBaseTransform * trans,
     GST_LOG_OBJECT (trans, "applying transformation to existing affine "
         "transformation meta");
 
-    gst_gl_get_affine_transformation_meta_as_ndc_ext (af_meta, upstream);
+    gst_gl_get_affine_transformation_meta_as_ndc (af_meta, upstream);
 
     /* apply the transformation to the existing affine meta */
     graphene_matrix_init_from_float (&upstream_matrix, upstream);
@@ -807,7 +822,7 @@ gst_gl_transformation_prepare_output_buffer (GstBaseTransform * trans,
     graphene_matrix_multiply (&tmp, &yflip, &tmp2);
 
     graphene_matrix_to_float (&tmp2, downstream);
-    gst_gl_set_affine_transformation_meta_from_ndc_ext (af_meta, downstream);
+    gst_gl_set_affine_transformation_meta_from_ndc (af_meta, downstream);
 
     return GST_FLOW_OK;
   }
@@ -961,7 +976,8 @@ gst_gl_transformation_callback (gpointer stuff)
 
   if (gl->GenVertexArrays)
     gl->BindVertexArray (0);
-  _unbind_buffer (transformation);
+  else
+    _unbind_buffer (transformation);
 
   gst_gl_context_clear_shader (GST_GL_BASE_FILTER (filter)->context);
   transformation->caps_change = FALSE;
