@@ -384,7 +384,7 @@ gst_audio_src_ring_buffer_acquire (GstAudioRingBuffer * buf,
   buf->size = spec->segtotal * spec->segsize;
   buf->memory = g_malloc (buf->size);
   if (buf->spec.type == GST_AUDIO_RING_BUFFER_FORMAT_TYPE_RAW) {
-    gst_audio_format_fill_silence (buf->spec.info.finfo, buf->memory,
+    gst_audio_format_info_fill_silence (buf->spec.info.finfo, buf->memory,
         buf->size);
   } else {
     /* FIXME, non-raw formats get 0 as the empty sample */
