@@ -58,7 +58,7 @@ GQuark gst_gl_base_memory_error_quark (void);
 
 /**
  * GstGLBaseMemoryError:
- * @GST_GL_BASE_MEMORY_ERROR_FAILED: generic faliure
+ * @GST_GL_BASE_MEMORY_ERROR_FAILED: generic failure
  * @GST_GL_BASE_MEMORY_ERROR_OLD_LIBS: the implementation is too old and doesn't
  *                                     implement enough features
  * @GST_GL_BASE_MEMORY_ERROR_RESOURCE_UNAVAILABLE: a resource could not be found
@@ -107,7 +107,7 @@ struct _GstGLBaseMemory
 
   GstGLContext         *context;
 
-  /* <protected> */
+  /*< protected >*/
   GMutex                lock;
 
   GstMapFlags           map_flags;       /* cumulative map flags */
@@ -118,7 +118,7 @@ struct _GstGLBaseMemory
 
   GstGLQuery           *query;
 
-  /* <private> */
+  /*< private >*/
   gsize                 alloc_size;     /* because maxsize is used for mapping */
   gpointer              alloc_data;
 
@@ -215,7 +215,7 @@ struct _GstGLAllocationParams
   /* GST_GL_ALLOCATION_PARAMS_ALLOC_FLAG_WRAP_GPU_HANDLE only */
   gpointer                          gl_handle;
 
-  /* <private> */
+  /*< private >*/
   gpointer                          _padding[GST_PADDING];
 };
 
@@ -227,7 +227,7 @@ gboolean                gst_gl_allocation_params_init       (GstGLAllocationPara
                                                              GstGLAllocationParamsFreeFunc free,
                                                              GstGLContext * context,
                                                              gsize alloc_size,
-                                                             GstAllocationParams * alloc_params,
+                                                             const GstAllocationParams * alloc_params,
                                                              gpointer wrapped_data,
                                                              gpointer gl_handle,
                                                              gpointer user_data,
@@ -370,8 +370,8 @@ struct _GstGLBaseMemoryAllocatorClass
   GstGLBaseMemoryAllocatorUnmapFunction         unmap;
   GstGLBaseMemoryAllocatorCopyFunction          copy;
   GstGLBaseMemoryAllocatorDestroyFunction       destroy;
-  /* <private> */
 
+  /*< private >*/
   gpointer                                      _padding[GST_PADDING];
 };
 
@@ -398,7 +398,7 @@ void          gst_gl_base_memory_init      (GstGLBaseMemory * mem,
                                             GstAllocator * allocator,
                                             GstMemory * parent,
                                             GstGLContext * context,
-                                            GstAllocationParams * params,
+                                            const GstAllocationParams * params,
                                             gsize size,
                                             gpointer user_data,
                                             GDestroyNotify notify);
