@@ -376,8 +376,8 @@ gst_gl_view_convert_set_context (GstGLViewConvert * viewconvert,
 
 static gboolean
 _view_convert_set_format (GstGLViewConvert * viewconvert,
-    GstVideoInfo * in_info, GstGLTextureTarget from_target,
-    GstVideoInfo * out_info, GstGLTextureTarget to_target)
+    const GstVideoInfo * in_info, GstGLTextureTarget from_target,
+    const GstVideoInfo * out_info, GstGLTextureTarget to_target)
 {
   gboolean passthrough;
   g_return_val_if_fail (GST_IS_GL_VIEW_CONVERT (viewconvert), FALSE);
@@ -1920,7 +1920,7 @@ _do_view_convert_draw (GstGLContext * context, GstGLViewConvert * viewconvert)
 
   gst_gl_shader_use (viewconvert->shader);
 
-  /* FIXME: the auxillary buffer could have a different transform matrix */
+  /* FIXME: the auxiliary buffer could have a different transform matrix */
   {
     GstVideoAffineTransformationMeta *af_meta;
     gfloat matrix[16];
@@ -1974,6 +1974,7 @@ _gen_buffer (GstGLViewConvert * viewconvert, GstBuffer ** target)
   GstGLVideoAllocationParams *params;
   GstGLMemoryAllocator *mem_allocator;
   GstAllocator *allocator;
+  GstVideoMeta *meta;
 
   *target = gst_buffer_new ();
 
@@ -1990,16 +1991,20 @@ _gen_buffer (GstGLViewConvert * viewconvert, GstBuffer ** target)
     gst_object_unref (allocator);
     return FALSE;
   }
-  gst_gl_allocation_params_free ((GstGLAllocationParams *) params);
   gst_object_unref (allocator);
 
-  gst_buffer_add_video_meta_full (*target, 0,
+  meta = gst_buffer_add_video_meta_full (*target, 0,
       GST_VIDEO_INFO_FORMAT (&viewconvert->out_info),
       GST_VIDEO_INFO_WIDTH (&viewconvert->out_info),
       GST_VIDEO_INFO_HEIGHT (&viewconvert->out_info),
       GST_VIDEO_INFO_N_PLANES (&viewconvert->out_info),
       viewconvert->out_info.offset, viewconvert->out_info.stride);
 
+  if (params->valign)
+    gst_video_meta_set_alignment (meta, *params->valign);
+
+  gst_gl_allocation_params_free ((GstGLAllocationParams *) params);
+
   return TRUE;
 }
 
@@ -2110,7 +2115,7 @@ _do_view_convert (GstGLContext * context, GstGLViewConvert * viewconvert)
         || out_tex->tex_format == GST_GL_LUMINANCE_ALPHA
         || out_width != width || out_height != height) {
       /* Luminance formats are not color renderable */
-      /* renderering to a framebuffer only renders the intersection of all
+      /* rendering to a framebuffer only renders the intersection of all
        * the attachments i.e. the smallest attachment size */
       if (!priv->out_tex[j]) {
         GstGLVideoAllocationParams *params;
@@ -2258,7 +2263,7 @@ gst_gl_view_convert_submit_input_buffer (GstGLViewConvert * viewconvert,
   target = &viewconvert->priv->primary_in;
 
   /* For frame-by-frame mode, we need to collect the 2nd eye into
-   * our auxilliary buffer */
+   * our auxiliary buffer */
   if (mode == GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME) {
     if (!GST_BUFFER_FLAG_IS_SET (input, GST_VIDEO_BUFFER_FLAG_FIRST_IN_BUNDLE))
       target = &viewconvert->priv->auxilliary_in;
