@@ -26,10 +26,11 @@
 
 #include "video-orc.h"
 #include "video-format.h"
-
+#include <gst/video/video-enumtypes.h>
 
 /**
  * SECTION:gstvideochroma
+ * @title: GstVideoChromaResample
  * @short_description: Functions and utility object for operating on chroma video planes
  *
  * The functions gst_video_chroma_from_string() and gst_video_chroma_to_string() convert
@@ -71,7 +72,9 @@ typedef struct
 static const ChromaSiteInfo chromasite[] = {
   {"jpeg", GST_VIDEO_CHROMA_SITE_JPEG},
   {"mpeg2", GST_VIDEO_CHROMA_SITE_MPEG2},
-  {"dv", GST_VIDEO_CHROMA_SITE_DV}
+  {"dv", GST_VIDEO_CHROMA_SITE_DV},
+  {"alt-line", GST_VIDEO_CHROMA_SITE_ALT_LINE},
+  {"cosited", GST_VIDEO_CHROMA_SITE_COSITED},
 };
 
 /**
@@ -80,18 +83,66 @@ static const ChromaSiteInfo chromasite[] = {
  *
  * Convert @s to a #GstVideoChromaSite
  *
+ * Deprecated: 1.20: Use gst_video_chroma_site_from_string() instead.
+ *
  * Returns: a #GstVideoChromaSite or %GST_VIDEO_CHROMA_SITE_UNKNOWN when @s does
  * not contain a valid chroma description.
  */
 GstVideoChromaSite
 gst_video_chroma_from_string (const gchar * s)
+{
+  return gst_video_chroma_site_from_string (s);
+}
+
+/**
+ * gst_video_chroma_site_from_string:
+ * @s: a chromasite string
+ *
+ * Convert @s to a #GstVideoChromaSite
+ *
+ * Returns: a #GstVideoChromaSite or %GST_VIDEO_CHROMA_SITE_UNKNOWN when @s does
+ * not contain a valid chroma-site description.
+ *
+ * Since: 1.20
+ */
+GstVideoChromaSite
+gst_video_chroma_site_from_string (const gchar * s)
 {
   gint i;
+  gchar **split;
+  gchar **iter;
+  GstVideoChromaSite ret = GST_VIDEO_CHROMA_SITE_UNKNOWN;
+  GFlagsClass *klass;
+
   for (i = 0; i < G_N_ELEMENTS (chromasite); i++) {
     if (g_str_equal (chromasite[i].name, s))
       return chromasite[i].site;
   }
-  return GST_VIDEO_CHROMA_SITE_UNKNOWN;
+
+  klass = (GFlagsClass *) g_type_class_ref (GST_TYPE_VIDEO_CHROMA_SITE);
+  split = g_strsplit (s, "+", 0);
+  for (iter = split; *iter; iter++) {
+    GFlagsValue *value;
+
+    value = g_flags_get_value_by_nick (klass, *iter);
+    if (!value) {
+      ret = GST_VIDEO_CHROMA_SITE_UNKNOWN;
+      goto out;
+    }
+
+    ret |= value->value;
+  }
+
+out:
+  g_type_class_unref (klass);
+  g_strfreev (split);
+
+  /* Doesn't make sense */
+  if ((ret & GST_VIDEO_CHROMA_SITE_NONE) != 0 &&
+      ret != GST_VIDEO_CHROMA_SITE_NONE)
+    return GST_VIDEO_CHROMA_SITE_UNKNOWN;
+
+  return ret;
 }
 
 /**
@@ -100,6 +151,8 @@ gst_video_chroma_from_string (const gchar * s)
  *
  * Converts @site to its string representation.
  *
+ * Deprecated: 1.20: Use gst_video_chroma_site_to_string() instead.
+ *
  * Returns: a string describing @site.
  */
 const gchar *
@@ -113,6 +166,60 @@ gst_video_chroma_to_string (GstVideoChromaSite site)
   return NULL;
 }
 
+/**
+ * gst_video_chroma_site_to_string:
+ * @site: a #GstVideoChromaSite
+ *
+ * Converts @site to its string representation.
+ *
+ * Returns: (transfer full) (nullable): a string representation of @site
+ *          or %NULL if @site contains undefined value or
+ *          is equal to %GST_VIDEO_CHROMA_SITE_UNKNOWN
+ *
+ * Since: 1.20
+ */
+gchar *
+gst_video_chroma_site_to_string (GstVideoChromaSite site)
+{
+  gint i;
+  GString *str;
+  GFlagsValue *value;
+  GFlagsClass *klass;
+
+  /* return null string for GST_VIDEO_CHROMA_SITE_UNKNOWN */
+  if (site == 0)
+    return NULL;
+
+  for (i = 0; i < G_N_ELEMENTS (chromasite); i++) {
+    if (chromasite[i].site == site)
+      return g_strdup (chromasite[i].name);
+  }
+
+  /* Doesn't make sense */
+  if ((site & GST_VIDEO_CHROMA_SITE_NONE) != 0 &&
+      site != GST_VIDEO_CHROMA_SITE_NONE)
+    return NULL;
+
+  /* Construct new string */
+  klass = (GFlagsClass *) g_type_class_ref (GST_TYPE_VIDEO_CHROMA_SITE);
+  str = g_string_new (NULL);
+  while (site != GST_VIDEO_CHROMA_SITE_UNKNOWN &&
+      (value = g_flags_get_first_value (klass, site))) {
+    if (str->len > 0)
+      g_string_append (str, "+");
+
+    g_string_append (str, value->value_nick);
+    site &= ~value->value;
+  }
+  g_type_class_unref (klass);
+
+  /* This means given chroma-site has unknown value */
+  if (site != 0)
+    return g_string_free (str, TRUE);
+
+  return g_string_free (str, FALSE);
+}
+
 struct _GstVideoChromaResample
 {
   GstVideoChromaMethod method;
@@ -213,7 +320,9 @@ video_chroma_up_v2_##name (GstVideoChromaResample *resample,            \
       resample->h_resample (resample, l1, width);                       \
   }                                                                     \
   if (l0 != l1) {                                                       \
-    video_orc_chroma_up_v2_##name (l0, l1, l0, l1, width);              \
+    type *d0 = l0;							\
+    type *d1 = l1;							\
+    video_orc_chroma_up_v2_##name (d0, d1, l0, l1, width);              \
   }                                                                     \
 }
 /* 2x vertical upsampling interlaced without cositing
@@ -290,8 +399,9 @@ video_chroma_down_h2_##name (GstVideoChromaResample *resample,          \
     gpointer pixels, gint width)                                        \
 {                                                                       \
   type *p = pixels;                                                     \
+  type *d = p;								\
                                                                         \
-  video_orc_chroma_down_h2_##name (p, p, width / 2);                    \
+  video_orc_chroma_down_h2_##name (d, p, width / 2);                    \
 }
 
 #define MAKE_DOWNSAMPLE_H2(name,type)                                   \
@@ -327,8 +437,10 @@ video_chroma_down_v2_##name (GstVideoChromaResample *resample,          \
   type *l0 = lines[0];                                                  \
   type *l1 = lines[1];                                                  \
                                                                         \
-  if (l0 != l1)                                                         \
-    video_orc_chroma_down_v2_##name (l0, l0, l1, width);                \
+  if (l0 != l1) {							\
+    type *d0 = l0;							\
+    video_orc_chroma_down_v2_##name (d0, l0, l1, width);                \
+  }									\
                                                                         \
   if (resample->h_resample)                                             \
     resample->h_resample (resample, l0, width);                         \
@@ -519,8 +631,9 @@ video_chroma_down_v4_##name (GstVideoChromaResample *resample,          \
   type *l1 = lines[1];                                                  \
   type *l2 = lines[2];                                                  \
   type *l3 = lines[3];                                                  \
-                                                                        \
-  video_orc_chroma_down_v4_##name(l0, l0, l1, l2, l3, width);           \
+  type *d = l0;								\
+									\
+  video_orc_chroma_down_v4_##name(d, l0, l1, l2, l3, width);		\
                                                                         \
   if (resample->h_resample)                                             \
     resample->h_resample (resample, l0, width);                         \
