@@ -165,12 +165,40 @@ gst_compositor_background_get_type (void)
   return compositor_background_type;
 }
 
+#define GST_TYPE_COMPOSITOR_SIZING_POLICY (gst_compositor_sizing_policy_get_type())
+static GType
+gst_compositor_sizing_policy_get_type (void)
+{
+  static GType sizing_policy_type = 0;
+
+  static const GEnumValue sizing_polices[] = {
+    {COMPOSITOR_SIZING_POLICY_NONE,
+        "None: Image is scaled to fill configured destination rectangle without "
+          "padding or keeping the aspect ratio", "none"},
+    {COMPOSITOR_SIZING_POLICY_KEEP_ASPECT_RATIO,
+          "Keep Aspect Ratio: Image is scaled to fit destination rectangle "
+          "specified by GstCompositorPad:{xpos, ypos, width, height} "
+          "with preserved aspect ratio. Resulting image will be centered in "
+          "the destination rectangle with padding if necessary",
+        "keep-aspect-ratio"},
+    {0, NULL, NULL},
+  };
+
+  if (!sizing_policy_type) {
+    sizing_policy_type =
+        g_enum_register_static ("GstCompositorSizingPolicy", sizing_polices);
+  }
+  return sizing_policy_type;
+}
+
 #define DEFAULT_PAD_XPOS   0
 #define DEFAULT_PAD_YPOS   0
-#define DEFAULT_PAD_WIDTH  0
-#define DEFAULT_PAD_HEIGHT 0
+#define DEFAULT_PAD_WIDTH  -1
+#define DEFAULT_PAD_HEIGHT -1
 #define DEFAULT_PAD_ALPHA  1.0
 #define DEFAULT_PAD_OPERATOR COMPOSITOR_OPERATOR_OVER
+#define DEFAULT_PAD_SIZING_POLICY COMPOSITOR_SIZING_POLICY_NONE
+
 enum
 {
   PROP_PAD_0,
@@ -180,10 +208,11 @@ enum
   PROP_PAD_HEIGHT,
   PROP_PAD_ALPHA,
   PROP_PAD_OPERATOR,
+  PROP_PAD_SIZING_POLICY,
 };
 
 G_DEFINE_TYPE (GstCompositorPad, gst_compositor_pad,
-    GST_TYPE_VIDEO_AGGREGATOR_CONVERT_PAD);
+    GST_TYPE_VIDEO_AGGREGATOR_PARALLEL_CONVERT_PAD);
 
 static void
 gst_compositor_pad_get_property (GObject * object, guint prop_id,
@@ -210,6 +239,9 @@ gst_compositor_pad_get_property (GObject * object, guint prop_id,
     case PROP_PAD_OPERATOR:
       g_value_set_enum (value, pad->op);
       break;
+    case PROP_PAD_SIZING_POLICY:
+      g_value_set_enum (value, pad->sizing_policy);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -247,6 +279,11 @@ gst_compositor_pad_set_property (GObject * object, guint prop_id,
       gst_video_aggregator_pad_set_needs_alpha (GST_VIDEO_AGGREGATOR_PAD (pad),
           pad->op == COMPOSITOR_OPERATOR_ADD);
       break;
+    case PROP_PAD_SIZING_POLICY:
+      pad->sizing_policy = g_value_get_enum (value);
+      gst_video_aggregator_convert_pad_update_conversion_info
+          (GST_VIDEO_AGGREGATOR_CONVERT_PAD (pad));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -254,49 +291,131 @@ gst_compositor_pad_set_property (GObject * object, guint prop_id,
 }
 
 static void
-_mixer_pad_get_output_size (GstCompositorPad * comp_pad, gint out_par_n,
-    gint out_par_d, gint * width, gint * height)
+_mixer_pad_get_output_size (GstCompositor * comp, GstCompositorPad * comp_pad,
+    gint out_par_n, gint out_par_d, gint * width, gint * height,
+    gint * x_offset, gint * y_offset)
 {
   GstVideoAggregatorPad *vagg_pad = GST_VIDEO_AGGREGATOR_PAD (comp_pad);
   gint pad_width, pad_height;
   guint dar_n, dar_d;
 
+  *x_offset = 0;
+  *y_offset = 0;
+  *width = 0;
+  *height = 0;
+
   /* FIXME: Anything better we can do here? */
   if (!vagg_pad->info.finfo
       || vagg_pad->info.finfo->format == GST_VIDEO_FORMAT_UNKNOWN) {
     GST_DEBUG_OBJECT (comp_pad, "Have no caps yet");
-    *width = 0;
-    *height = 0;
     return;
   }
 
-  pad_width =
-      comp_pad->width <=
-      0 ? GST_VIDEO_INFO_WIDTH (&vagg_pad->info) : comp_pad->width;
-  pad_height =
-      comp_pad->height <=
-      0 ? GST_VIDEO_INFO_HEIGHT (&vagg_pad->info) : comp_pad->height;
+  if (comp->zero_size_is_unscaled) {
+    pad_width =
+        comp_pad->width <=
+        0 ? GST_VIDEO_INFO_WIDTH (&vagg_pad->info) : comp_pad->width;
+    pad_height =
+        comp_pad->height <=
+        0 ? GST_VIDEO_INFO_HEIGHT (&vagg_pad->info) : comp_pad->height;
+  } else {
+    pad_width =
+        comp_pad->width <
+        0 ? GST_VIDEO_INFO_WIDTH (&vagg_pad->info) : comp_pad->width;
+    pad_height =
+        comp_pad->height <
+        0 ? GST_VIDEO_INFO_HEIGHT (&vagg_pad->info) : comp_pad->height;
+  }
+
+  if (pad_width == 0 || pad_height == 0)
+    return;
 
   if (!gst_video_calculate_display_ratio (&dar_n, &dar_d, pad_width, pad_height,
           GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
           GST_VIDEO_INFO_PAR_D (&vagg_pad->info), out_par_n, out_par_d)) {
     GST_WARNING_OBJECT (comp_pad, "Cannot calculate display aspect ratio");
-    *width = *height = 0;
     return;
   }
+
   GST_LOG_OBJECT (comp_pad, "scaling %ux%u by %u/%u (%u/%u / %u/%u)", pad_width,
       pad_height, dar_n, dar_d, GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
       GST_VIDEO_INFO_PAR_D (&vagg_pad->info), out_par_n, out_par_d);
 
-  /* Pick either height or width, whichever is an integer multiple of the
-   * display aspect ratio. However, prefer preserving the height to account
-   * for interlaced video. */
-  if (pad_height % dar_n == 0) {
-    pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
-  } else if (pad_width % dar_d == 0) {
-    pad_height = gst_util_uint64_scale_int (pad_width, dar_d, dar_n);
-  } else {
-    pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+  switch (comp_pad->sizing_policy) {
+    case COMPOSITOR_SIZING_POLICY_NONE:
+      /* Pick either height or width, whichever is an integer multiple of the
+       * display aspect ratio. However, prefer preserving the height to account
+       * for interlaced video. */
+      if (pad_height % dar_n == 0) {
+        pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+      } else if (pad_width % dar_d == 0) {
+        pad_height = gst_util_uint64_scale_int (pad_width, dar_d, dar_n);
+      } else {
+        pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
+      }
+      break;
+    case COMPOSITOR_SIZING_POLICY_KEEP_ASPECT_RATIO:
+    {
+      gint from_dar_n, from_dar_d, to_dar_n, to_dar_d, num, den;
+
+      /* Calculate DAR again with actual video size */
+      if (!gst_util_fraction_multiply (GST_VIDEO_INFO_WIDTH (&vagg_pad->info),
+              GST_VIDEO_INFO_HEIGHT (&vagg_pad->info),
+              GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
+              GST_VIDEO_INFO_PAR_D (&vagg_pad->info), &from_dar_n,
+              &from_dar_d)) {
+        from_dar_n = from_dar_d = -1;
+      }
+
+      if (!gst_util_fraction_multiply (pad_width, pad_height,
+              out_par_n, out_par_d, &to_dar_n, &to_dar_d)) {
+        to_dar_n = to_dar_d = -1;
+      }
+
+      if (from_dar_n != to_dar_n || from_dar_d != to_dar_d) {
+        /* Calculate new output resolution */
+        if (from_dar_n != -1 && from_dar_d != -1
+            && gst_util_fraction_multiply (from_dar_n, from_dar_d,
+                out_par_d, out_par_n, &num, &den)) {
+          GstVideoRectangle src_rect, dst_rect, rst_rect;
+
+          src_rect.h = gst_util_uint64_scale_int (pad_width, den, num);
+          if (src_rect.h == 0) {
+            pad_width = 0;
+            pad_height = 0;
+            break;
+          }
+
+          src_rect.x = src_rect.y = 0;
+          src_rect.w = pad_width;
+
+          dst_rect.x = dst_rect.y = 0;
+          dst_rect.w = pad_width;
+          dst_rect.h = pad_height;
+
+          /* Scale rect to be centered in destination rect */
+          gst_video_center_rect (&src_rect, &dst_rect, &rst_rect, TRUE);
+
+          GST_LOG_OBJECT (comp_pad,
+              "Re-calculated size %dx%d -> %dx%d (x-offset %d, y-offset %d)",
+              pad_width, pad_height, rst_rect.w, rst_rect.h, rst_rect.x,
+              rst_rect.h);
+
+          *x_offset = rst_rect.x;
+          *y_offset = rst_rect.y;
+          pad_width = rst_rect.w;
+          pad_height = rst_rect.h;
+        } else {
+          GST_WARNING_OBJECT (comp_pad, "Failed to calculate output size");
+
+          *x_offset = 0;
+          *y_offset = 0;
+          pad_width = 0;
+          pad_height = 0;
+        }
+      }
+      break;
+    }
   }
 
   *width = pad_width;
@@ -342,6 +461,10 @@ _pad_obscures_rectangle (GstVideoAggregator * vagg, GstVideoAggregatorPad * pad,
 {
   GstVideoRectangle pad_rect;
   GstCompositorPad *cpad = GST_COMPOSITOR_PAD (pad);
+  GstStructure *converter_config = NULL;
+  gboolean fill_border = TRUE;
+  guint32 border_argb = 0xff000000;
+  gint x_offset, y_offset;
 
   /* No buffer to obscure the rectangle with */
   if (!gst_video_aggregator_pad_has_current_buffer (pad))
@@ -354,11 +477,28 @@ _pad_obscures_rectangle (GstVideoAggregator * vagg, GstVideoAggregatorPad * pad,
   if (cpad->alpha != 1.0 || GST_VIDEO_INFO_HAS_ALPHA (&pad->info))
     return FALSE;
 
+  /* If a converter-config is set and it is either configured to not fill any
+   * borders, or configured to use a non-opaque color, then we have to handle
+   * the pad as potentially containing transparency */
+  g_object_get (pad, "converter-config", &converter_config, NULL);
+  if (converter_config) {
+    gst_structure_get (converter_config, GST_VIDEO_CONVERTER_OPT_BORDER_ARGB,
+        G_TYPE_UINT, &border_argb, NULL);
+    gst_structure_get (converter_config, GST_VIDEO_CONVERTER_OPT_FILL_BORDER,
+        G_TYPE_BOOLEAN, &fill_border, NULL);
+  }
+  gst_clear_structure (&converter_config);
+  if (!fill_border || (border_argb & 0xff000000) != 0xff000000)
+    return FALSE;
+
   pad_rect.x = cpad->xpos;
   pad_rect.y = cpad->ypos;
   /* Handle pixel and display aspect ratios to find the actual size */
-  _mixer_pad_get_output_size (cpad, GST_VIDEO_INFO_PAR_N (&vagg->info),
-      GST_VIDEO_INFO_PAR_D (&vagg->info), &(pad_rect.w), &(pad_rect.h));
+  _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), cpad,
+      GST_VIDEO_INFO_PAR_N (&vagg->info), GST_VIDEO_INFO_PAR_D (&vagg->info),
+      &(pad_rect.w), &(pad_rect.h), &x_offset, &y_offset);
+  pad_rect.x += x_offset;
+  pad_rect.y += y_offset;
 
   if (!is_rectangle_contained (rect, pad_rect))
     return FALSE;
@@ -370,8 +510,8 @@ _pad_obscures_rectangle (GstVideoAggregator * vagg, GstVideoAggregatorPad * pad,
   return TRUE;
 }
 
-static gboolean
-gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
+static void
+gst_compositor_pad_prepare_frame_start (GstVideoAggregatorPad * pad,
     GstVideoAggregator * vagg, GstBuffer * buffer,
     GstVideoFrame * prepared_frame)
 {
@@ -395,21 +535,26 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
    *     width/height. See ->set_info()
    * */
 
-  _mixer_pad_get_output_size (cpad, GST_VIDEO_INFO_PAR_N (&vagg->info),
-      GST_VIDEO_INFO_PAR_D (&vagg->info), &width, &height);
+  _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), cpad,
+      GST_VIDEO_INFO_PAR_N (&vagg->info), GST_VIDEO_INFO_PAR_D (&vagg->info),
+      &width, &height, &cpad->x_offset, &cpad->y_offset);
 
   if (cpad->alpha == 0.0) {
     GST_DEBUG_OBJECT (pad, "Pad has alpha 0.0, not converting frame");
-    goto done;
+    return;
   }
 
-  frame_rect = clamp_rectangle (cpad->xpos, cpad->ypos, width, height,
+  if (gst_aggregator_pad_is_inactive (GST_AGGREGATOR_PAD (pad)))
+    return;
+
+  frame_rect = clamp_rectangle (cpad->xpos + cpad->x_offset,
+      cpad->ypos + cpad->y_offset, width, height,
       GST_VIDEO_INFO_WIDTH (&vagg->info), GST_VIDEO_INFO_HEIGHT (&vagg->info));
 
   if (frame_rect.w == 0 || frame_rect.h == 0) {
     GST_DEBUG_OBJECT (pad, "Resulting frame is zero-width or zero-height "
         "(w: %i, h: %i), skipping", frame_rect.w, frame_rect.h);
-    goto done;
+    return;
   }
 
   GST_OBJECT_LOCK (vagg);
@@ -429,16 +574,11 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
   GST_OBJECT_UNLOCK (vagg);
 
   if (frame_obscured)
-    goto done;
+    return;
 
-  return
-      GST_VIDEO_AGGREGATOR_PAD_CLASS
-      (gst_compositor_pad_parent_class)->prepare_frame (pad, vagg, buffer,
+  GST_VIDEO_AGGREGATOR_PAD_CLASS
+      (gst_compositor_pad_parent_class)->prepare_frame_start (pad, vagg, buffer,
       prepared_frame);
-
-done:
-
-  return TRUE;
 }
 
 static void
@@ -447,6 +587,7 @@ gst_compositor_pad_create_conversion_info (GstVideoAggregatorConvertPad * pad,
 {
   GstCompositorPad *cpad = GST_COMPOSITOR_PAD (pad);
   gint width, height;
+  gint x_offset, y_offset;
 
   GST_VIDEO_AGGREGATOR_CONVERT_PAD_CLASS
       (gst_compositor_pad_parent_class)->create_conversion_info (pad, vagg,
@@ -454,8 +595,9 @@ gst_compositor_pad_create_conversion_info (GstVideoAggregatorConvertPad * pad,
   if (!conversion_info->finfo)
     return;
 
-  _mixer_pad_get_output_size (cpad, GST_VIDEO_INFO_PAR_N (&vagg->info),
-      GST_VIDEO_INFO_PAR_D (&vagg->info), &width, &height);
+  _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), cpad,
+      GST_VIDEO_INFO_PAR_N (&vagg->info), GST_VIDEO_INFO_PAR_D (&vagg->info),
+      &width, &height, &x_offset, &y_offset);
 
   /* The only thing that can change here is the width
    * and height, otherwise set_info would've been called */
@@ -521,11 +663,29 @@ gst_compositor_pad_class_init (GstCompositorPadClass * klass)
           GST_TYPE_COMPOSITOR_OPERATOR, DEFAULT_PAD_OPERATOR,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 
-  vaggpadclass->prepare_frame =
-      GST_DEBUG_FUNCPTR (gst_compositor_pad_prepare_frame);
+  /**
+   * GstCompositorPad:sizing-policy:
+   *
+   * Specifies sizing policy to use. Depending on selected sizing policy,
+   * scaled image might not fully cover the configured target rectangle area
+   * (e.g., "keep-aspect-ratio"). In that case, any uncovered area will be
+   * filled with background unless the uncovered area is drawn by other image.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_PAD_SIZING_POLICY,
+      g_param_spec_enum ("sizing-policy", "Sizing policy",
+          "Sizing policy to use for image scaling",
+          GST_TYPE_COMPOSITOR_SIZING_POLICY, DEFAULT_PAD_SIZING_POLICY,
+          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
+
+  vaggpadclass->prepare_frame_start =
+      GST_DEBUG_FUNCPTR (gst_compositor_pad_prepare_frame_start);
 
   vaggcpadclass->create_conversion_info =
       GST_DEBUG_FUNCPTR (gst_compositor_pad_create_conversion_info);
+
+  gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_SIZING_POLICY, 0);
 }
 
 static void
@@ -535,15 +695,24 @@ gst_compositor_pad_init (GstCompositorPad * compo_pad)
   compo_pad->ypos = DEFAULT_PAD_YPOS;
   compo_pad->alpha = DEFAULT_PAD_ALPHA;
   compo_pad->op = DEFAULT_PAD_OPERATOR;
+  compo_pad->width = DEFAULT_PAD_WIDTH;
+  compo_pad->height = DEFAULT_PAD_HEIGHT;
+  compo_pad->sizing_policy = DEFAULT_PAD_SIZING_POLICY;
 }
 
 
 /* GstCompositor */
 #define DEFAULT_BACKGROUND COMPOSITOR_BACKGROUND_CHECKER
+#define DEFAULT_ZERO_SIZE_IS_UNSCALED TRUE
+#define DEFAULT_MAX_THREADS 0
+
 enum
 {
   PROP_0,
   PROP_BACKGROUND,
+  PROP_ZERO_SIZE_IS_UNSCALED,
+  PROP_MAX_THREADS,
+  PROP_IGNORE_INACTIVE_PADS,
 };
 
 static void
@@ -556,6 +725,16 @@ gst_compositor_get_property (GObject * object,
     case PROP_BACKGROUND:
       g_value_set_enum (value, self->background);
       break;
+    case PROP_ZERO_SIZE_IS_UNSCALED:
+      g_value_set_boolean (value, self->zero_size_is_unscaled);
+      break;
+    case PROP_MAX_THREADS:
+      g_value_set_uint (value, self->max_threads);
+      break;
+    case PROP_IGNORE_INACTIVE_PADS:
+      g_value_set_boolean (value,
+          gst_aggregator_get_ignore_inactive_pads (GST_AGGREGATOR (object)));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -572,6 +751,16 @@ gst_compositor_set_property (GObject * object,
     case PROP_BACKGROUND:
       self->background = g_value_get_enum (value);
       break;
+    case PROP_ZERO_SIZE_IS_UNSCALED:
+      self->zero_size_is_unscaled = g_value_get_boolean (value);
+      break;
+    case PROP_MAX_THREADS:
+      self->max_threads = g_value_get_uint (value);
+      break;
+    case PROP_IGNORE_INACTIVE_PADS:
+      gst_aggregator_set_ignore_inactive_pads (GST_AGGREGATOR (object),
+          g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -582,9 +771,11 @@ gst_compositor_set_property (GObject * object,
 G_DEFINE_TYPE_WITH_CODE (GstCompositor, gst_compositor,
     GST_TYPE_VIDEO_AGGREGATOR, G_IMPLEMENT_INTERFACE (GST_TYPE_CHILD_PROXY,
         gst_compositor_child_proxy_init));
+GST_ELEMENT_REGISTER_DEFINE (compositor, "compositor", GST_RANK_PRIMARY + 1,
+    GST_TYPE_COMPOSITOR);
 
 static gboolean
-set_functions (GstCompositor * self, GstVideoInfo * info)
+set_functions (GstCompositor * self, const GstVideoInfo * info)
 {
   gboolean ret = FALSE;
 
@@ -786,16 +977,25 @@ _fixate_caps (GstAggregator * agg, GstCaps * caps)
     gint width, height;
     gint fps_n, fps_d;
     gdouble cur_fps;
+    gint x_offset;
+    gint y_offset;
+
+    if (gst_aggregator_pad_is_inactive (GST_AGGREGATOR_PAD (vaggpad)))
+      continue;
 
     fps_n = GST_VIDEO_INFO_FPS_N (&vaggpad->info);
     fps_d = GST_VIDEO_INFO_FPS_D (&vaggpad->info);
-    _mixer_pad_get_output_size (compositor_pad, par_n, par_d, &width, &height);
+    _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), compositor_pad, par_n,
+        par_d, &width, &height, &x_offset, &y_offset);
 
     if (width == 0 || height == 0)
       continue;
 
-    this_width = width + MAX (compositor_pad->xpos, 0);
-    this_height = height + MAX (compositor_pad->ypos, 0);
+    /* {x,y}_offset represent padding size of each top and left area.
+     * To calculate total resolution, count bottom and right padding area
+     * as well here */
+    this_width = width + MAX (compositor_pad->xpos + 2 * x_offset, 0);
+    this_height = height + MAX (compositor_pad->ypos + 2 * y_offset, 0);
 
     if (best_width < this_width)
       best_width = this_width;
@@ -830,21 +1030,190 @@ _fixate_caps (GstAggregator * agg, GstCaps * caps)
   return ret;
 }
 
+static void
+gst_parallelized_task_thread_func (gpointer data)
+{
+  GstParallelizedTaskRunner *runner = data;
+  gint idx;
+
+  g_mutex_lock (&runner->lock);
+  idx = runner->n_todo--;
+  g_assert (runner->n_todo >= -1);
+  g_mutex_unlock (&runner->lock);
+
+  g_assert (runner->func != NULL);
+
+  runner->func (runner->task_data[idx]);
+}
+
+static void
+gst_parallelized_task_runner_join (GstParallelizedTaskRunner * self)
+{
+  gboolean joined = FALSE;
+
+  while (!joined) {
+    g_mutex_lock (&self->lock);
+    if (!(joined = gst_queue_array_is_empty (self->tasks))) {
+      gpointer task = gst_queue_array_pop_head (self->tasks);
+      g_mutex_unlock (&self->lock);
+      gst_task_pool_join (self->pool, task);
+    } else {
+      g_mutex_unlock (&self->lock);
+    }
+  }
+}
+
+static void
+gst_parallelized_task_runner_free (GstParallelizedTaskRunner * self)
+{
+  gst_parallelized_task_runner_join (self);
+
+  gst_queue_array_free (self->tasks);
+  if (self->own_pool)
+    gst_task_pool_cleanup (self->pool);
+  gst_object_unref (self->pool);
+  g_mutex_clear (&self->lock);
+  g_free (self);
+}
+
+static GstParallelizedTaskRunner *
+gst_parallelized_task_runner_new (guint n_threads, GstTaskPool * pool,
+    gboolean async_tasks)
+{
+  GstParallelizedTaskRunner *self;
+
+  if (n_threads == 0)
+    n_threads = g_get_num_processors ();
+
+  self = g_new0 (GstParallelizedTaskRunner, 1);
+
+  if (pool) {
+    self->pool = g_object_ref (pool);
+    self->own_pool = FALSE;
+
+    /* No reason to split up the work between more threads than the
+     * pool can spawn */
+    if (GST_IS_SHARED_TASK_POOL (pool))
+      n_threads =
+          MIN (n_threads,
+          gst_shared_task_pool_get_max_threads (GST_SHARED_TASK_POOL (pool)));
+  } else {
+    self->pool = gst_shared_task_pool_new ();
+    self->own_pool = TRUE;
+    gst_shared_task_pool_set_max_threads (GST_SHARED_TASK_POOL (self->pool),
+        n_threads);
+    gst_task_pool_prepare (self->pool, NULL);
+  }
+
+  self->tasks = gst_queue_array_new (n_threads);
+
+  self->n_threads = n_threads;
+
+  self->n_todo = -1;
+  g_mutex_init (&self->lock);
+
+  /* Set when scheduling a job */
+  self->func = NULL;
+  self->task_data = NULL;
+  self->async_tasks = async_tasks;
+
+  return self;
+}
+
+static void
+gst_parallelized_task_runner_finish (GstParallelizedTaskRunner * self)
+{
+  g_return_if_fail (self->func != NULL);
+
+  gst_parallelized_task_runner_join (self);
+
+  self->func = NULL;
+  self->task_data = NULL;
+}
+
+static void
+gst_parallelized_task_runner_run (GstParallelizedTaskRunner * self,
+    GstParallelizedTaskFunc func, gpointer * task_data)
+{
+  guint n_threads = self->n_threads;
+
+  self->func = func;
+  self->task_data = task_data;
+
+  if (n_threads > 1 || self->async_tasks) {
+    guint i = 0;
+    g_mutex_lock (&self->lock);
+    self->n_todo = self->n_threads - 1;
+    if (!self->async_tasks) {
+      /* if not async, perform one of the functions in the current thread */
+      self->n_todo--;
+      i = 1;
+    }
+    for (; i < n_threads; i++) {
+      gpointer task =
+          gst_task_pool_push (self->pool, gst_parallelized_task_thread_func,
+          self, NULL);
+
+      /* The return value of push() is nullable but NULL is only returned
+       * with the shared task pool when gst_task_pool_prepare() has not been
+       * called and would thus be a programming error that we should hard-fail
+       * on.
+       */
+      g_assert (task != NULL);
+      gst_queue_array_push_tail (self->tasks, task);
+    }
+    g_mutex_unlock (&self->lock);
+  }
+
+  if (!self->async_tasks) {
+    self->func (self->task_data[self->n_threads - 1]);
+
+    gst_parallelized_task_runner_finish (self);
+  }
+}
+
 static gboolean
 _negotiated_caps (GstAggregator * agg, GstCaps * caps)
 {
+  GstCompositor *compositor = GST_COMPOSITOR (agg);
+  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (agg);
   GstVideoInfo v_info;
+  guint n_threads;
 
   GST_DEBUG_OBJECT (agg, "Negotiated caps %" GST_PTR_FORMAT, caps);
 
   if (!gst_video_info_from_caps (&v_info, caps))
     return FALSE;
 
-  if (!set_functions (GST_COMPOSITOR (agg), &v_info)) {
+  if (!set_functions (compositor, &v_info)) {
     GST_ERROR_OBJECT (agg, "Failed to setup vfuncs");
     return FALSE;
   }
 
+  if (compositor->max_threads == 0)
+    n_threads = g_get_num_processors ();
+  else
+    n_threads = compositor->max_threads;
+
+  /* Magic number of 200 lines */
+  if (GST_VIDEO_INFO_HEIGHT (&v_info) / n_threads < 200)
+    n_threads = (GST_VIDEO_INFO_HEIGHT (&v_info) + 199) / 200;
+  if (n_threads < 1)
+    n_threads = 1;
+
+  /* XXX: implement better thread count change */
+  if (compositor->blend_runner
+      && compositor->blend_runner->n_threads != n_threads) {
+    gst_parallelized_task_runner_free (compositor->blend_runner);
+    compositor->blend_runner = NULL;
+  }
+  if (!compositor->blend_runner) {
+    GstTaskPool *pool = gst_video_aggregator_get_execution_task_pool (vagg);
+    compositor->blend_runner =
+        gst_parallelized_task_runner_new (n_threads, pool, FALSE);
+    gst_clear_object (&pool);
+  }
+
   return GST_AGGREGATOR_CLASS (parent_class)->negotiated_src_caps (agg, caps);
 }
 
@@ -863,6 +1232,9 @@ _should_draw_background (GstVideoAggregator * vagg)
   /* Check if the background is completely obscured by a pad
    * TODO: Also skip if it's obscured by a combination of pads */
   for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
+    if (gst_aggregator_pad_is_inactive (GST_AGGREGATOR_PAD (l->data)))
+      continue;
+
     if (_pad_obscures_rectangle (vagg, l->data, bg_rect)) {
       draw = FALSE;
       break;
@@ -872,58 +1244,6 @@ _should_draw_background (GstVideoAggregator * vagg)
   return draw;
 }
 
-static gboolean
-_draw_background (GstVideoAggregator * vagg, GstVideoFrame * outframe,
-    BlendFunction * composite)
-{
-  GstCompositor *comp = GST_COMPOSITOR (vagg);
-
-  *composite = comp->blend;
-  /* If one of the frames to be composited completely obscures the background,
-   * don't bother drawing the background at all. We can also always use the
-   * 'blend' BlendFunction in that case because it only changes if we have to
-   * overlay on top of a transparent background. */
-  if (!_should_draw_background (vagg))
-    return FALSE;
-
-  switch (comp->background) {
-    case COMPOSITOR_BACKGROUND_CHECKER:
-      comp->fill_checker (outframe);
-      break;
-    case COMPOSITOR_BACKGROUND_BLACK:
-      comp->fill_color (outframe, 16, 128, 128);
-      break;
-    case COMPOSITOR_BACKGROUND_WHITE:
-      comp->fill_color (outframe, 240, 128, 128);
-      break;
-    case COMPOSITOR_BACKGROUND_TRANSPARENT:
-    {
-      guint i, plane, num_planes, height;
-
-      num_planes = GST_VIDEO_FRAME_N_PLANES (outframe);
-      for (plane = 0; plane < num_planes; ++plane) {
-        guint8 *pdata;
-        gsize rowsize, plane_stride;
-
-        pdata = GST_VIDEO_FRAME_PLANE_DATA (outframe, plane);
-        plane_stride = GST_VIDEO_FRAME_PLANE_STRIDE (outframe, plane);
-        rowsize = GST_VIDEO_FRAME_COMP_WIDTH (outframe, plane)
-            * GST_VIDEO_FRAME_COMP_PSTRIDE (outframe, plane);
-        height = GST_VIDEO_FRAME_COMP_HEIGHT (outframe, plane);
-        for (i = 0; i < height; ++i) {
-          memset (pdata, 0, rowsize);
-          pdata += plane_stride;
-        }
-      }
-      /* use overlay to keep background transparent */
-      *composite = comp->overlay;
-      break;
-    }
-  }
-
-  return TRUE;
-}
-
 static gboolean
 frames_can_copy (const GstVideoFrame * frame1, const GstVideoFrame * frame2)
 {
@@ -936,14 +1256,109 @@ frames_can_copy (const GstVideoFrame * frame1, const GstVideoFrame * frame2)
   return TRUE;
 }
 
+struct CompositePadInfo
+{
+  GstVideoFrame *prepared_frame;
+  GstCompositorPad *pad;
+  GstCompositorBlendMode blend_mode;
+};
+
+struct CompositeTask
+{
+  GstCompositor *compositor;
+  GstVideoFrame *out_frame;
+  guint dst_line_start;
+  guint dst_line_end;
+  gboolean draw_background;
+  guint n_pads;
+  struct CompositePadInfo *pads_info;
+};
+
+static void
+_draw_background (GstCompositor * comp, GstVideoFrame * outframe,
+    guint y_start, guint y_end, BlendFunction * composite)
+{
+  *composite = comp->blend;
+
+  switch (comp->background) {
+    case COMPOSITOR_BACKGROUND_CHECKER:
+      comp->fill_checker (outframe, y_start, y_end);
+      break;
+    case COMPOSITOR_BACKGROUND_BLACK:
+      comp->fill_color (outframe, y_start, y_end, 16, 128, 128);
+      break;
+    case COMPOSITOR_BACKGROUND_WHITE:
+      comp->fill_color (outframe, y_start, y_end, 240, 128, 128);
+      break;
+    case COMPOSITOR_BACKGROUND_TRANSPARENT:
+    {
+      guint i, plane, num_planes, height;
+
+      num_planes = GST_VIDEO_FRAME_N_PLANES (outframe);
+      for (plane = 0; plane < num_planes; ++plane) {
+        const GstVideoFormatInfo *info;
+        gint comp[GST_VIDEO_MAX_COMPONENTS];
+        guint8 *pdata;
+        gsize rowsize, plane_stride;
+        gint yoffset;
+
+        info = outframe->info.finfo;
+        pdata = GST_VIDEO_FRAME_PLANE_DATA (outframe, plane);
+        plane_stride = GST_VIDEO_FRAME_PLANE_STRIDE (outframe, plane);
+
+        gst_video_format_info_component (info, plane, comp);
+        rowsize = GST_VIDEO_FRAME_COMP_WIDTH (outframe, comp[0])
+            * GST_VIDEO_FRAME_COMP_PSTRIDE (outframe, comp[0]);
+        height = GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, comp[0],
+            (y_end - y_start));
+
+        yoffset = GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (info, comp[0], y_start);
+
+        pdata += yoffset * plane_stride;
+        for (i = 0; i < height; ++i) {
+          memset (pdata, 0, rowsize);
+          pdata += plane_stride;
+        }
+      }
+      /* use overlay to keep background transparent */
+      *composite = comp->overlay;
+      break;
+    }
+  }
+}
+
+static void
+blend_pads (struct CompositeTask *comp)
+{
+  BlendFunction composite;
+  guint i;
+
+  composite = comp->compositor->blend;
+
+  if (comp->draw_background) {
+    _draw_background (comp->compositor, comp->out_frame, comp->dst_line_start,
+        comp->dst_line_end, &composite);
+  }
+
+  for (i = 0; i < comp->n_pads; i++) {
+    composite (comp->pads_info[i].prepared_frame,
+        comp->pads_info[i].pad->xpos + comp->pads_info[i].pad->x_offset,
+        comp->pads_info[i].pad->ypos + comp->pads_info[i].pad->y_offset,
+        comp->pads_info[i].pad->alpha, comp->out_frame, comp->dst_line_start,
+        comp->dst_line_end, comp->pads_info[i].blend_mode);
+  }
+}
+
 static GstFlowReturn
 gst_compositor_aggregate_frames (GstVideoAggregator * vagg, GstBuffer * outbuf)
 {
+  GstCompositor *compositor = GST_COMPOSITOR (vagg);
   GList *l;
-  BlendFunction composite;
   GstVideoFrame out_frame, *outframe;
-  gboolean drew_background;
-  guint drawn_pads = 0;
+  gboolean draw_background;
+  guint drawn_a_pad = FALSE;
+  struct CompositePadInfo *pads_info;
+  guint i, n_pads = 0;
 
   if (!gst_video_frame_map (&out_frame, &vagg->info, outbuf, GST_MAP_WRITE)) {
     GST_WARNING_OBJECT (vagg, "Could not map output buffer");
@@ -951,9 +1366,31 @@ gst_compositor_aggregate_frames (GstVideoAggregator * vagg, GstBuffer * outbuf)
   }
 
   outframe = &out_frame;
-  drew_background = _draw_background (vagg, outframe, &composite);
+
+  /* If one of the frames to be composited completely obscures the background,
+   * don't bother drawing the background at all. We can also always use the
+   * 'blend' BlendFunction in that case because it only changes if we have to
+   * overlay on top of a transparent background. */
+  draw_background = _should_draw_background (vagg);
 
   GST_OBJECT_LOCK (vagg);
+  for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
+    GstVideoAggregatorPad *pad = l->data;
+    GstVideoFrame *prepared_frame =
+        gst_video_aggregator_pad_get_prepared_frame (pad);
+
+    if (prepared_frame)
+      n_pads++;
+  }
+
+  /* If no prepared frame, we should draw background unconditionally in order
+   * to clear output buffer */
+  if (n_pads == 0)
+    draw_background = TRUE;
+
+  pads_info = g_newa (struct CompositePadInfo, n_pads);
+  n_pads = 0;
+
   for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
     GstVideoAggregatorPad *pad = l->data;
     GstCompositorPad *compo_pad = GST_COMPOSITOR_PAD (pad);
@@ -981,16 +1418,53 @@ gst_compositor_aggregate_frames (GstVideoAggregator * vagg, GstBuffer * outbuf)
        * background, and @prepared_frame has the same format, height, and width
        * as @outframe, then we can just copy it as-is. Subsequent pads (if any)
        * will be composited on top of it. */
-      if (drawn_pads == 0 && !drew_background &&
-          frames_can_copy (prepared_frame, outframe))
+      if (!drawn_a_pad && !draw_background &&
+          frames_can_copy (prepared_frame, outframe)) {
         gst_video_frame_copy (outframe, prepared_frame);
-      else
-        composite (prepared_frame,
-            compo_pad->xpos,
-            compo_pad->ypos, compo_pad->alpha, outframe, blend_mode);
-      drawn_pads++;
+      } else {
+        pads_info[n_pads].pad = compo_pad;
+        pads_info[n_pads].prepared_frame = prepared_frame;
+        pads_info[n_pads].blend_mode = blend_mode;
+        n_pads++;
+      }
+      drawn_a_pad = TRUE;
     }
   }
+
+  {
+    guint n_threads, lines_per_thread;
+    guint out_height;
+    struct CompositeTask *tasks;
+    struct CompositeTask **tasks_p;
+
+    n_threads = compositor->blend_runner->n_threads;
+
+    tasks = g_newa (struct CompositeTask, n_threads);
+    tasks_p = g_newa (struct CompositeTask *, n_threads);
+
+    out_height = GST_VIDEO_FRAME_HEIGHT (outframe);
+    lines_per_thread = (out_height + n_threads - 1) / n_threads;
+
+    for (i = 0; i < n_threads; i++) {
+      tasks[i].compositor = compositor;
+      tasks[i].n_pads = n_pads;
+      tasks[i].pads_info = pads_info;
+      tasks[i].out_frame = outframe;
+      tasks[i].draw_background = draw_background;
+      /* This is a dumb split of the work by number of output lines.
+       * If there is a section of the output that reads from a lot of source
+       * pads, then that thread will consume more time. Maybe tracking and
+       * splitting on the source fill rate would produce better results. */
+      tasks[i].dst_line_start = i * lines_per_thread;
+      tasks[i].dst_line_end = MIN ((i + 1) * lines_per_thread, out_height);
+
+      tasks_p[i] = &tasks[i];
+    }
+
+    gst_parallelized_task_runner_run (compositor->blend_runner,
+        (GstParallelizedTaskFunc) blend_pads, (gpointer *) tasks_p);
+  }
+
   GST_OBJECT_UNLOCK (vagg);
 
   gst_video_frame_unmap (outframe);
@@ -1080,6 +1554,18 @@ _sink_query (GstAggregator * agg, GstAggregatorPad * bpad, GstQuery * query)
   }
 }
 
+static void
+gst_compositor_finalize (GObject * object)
+{
+  GstCompositor *compositor = GST_COMPOSITOR (object);
+
+  if (compositor->blend_runner)
+    gst_parallelized_task_runner_free (compositor->blend_runner);
+  compositor->blend_runner = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 /* GObject boilerplate */
 static void
 gst_compositor_class_init (GstCompositorClass * klass)
@@ -1092,6 +1578,7 @@ gst_compositor_class_init (GstCompositorClass * klass)
 
   gobject_class->get_property = gst_compositor_get_property;
   gobject_class->set_property = gst_compositor_set_property;
+  gobject_class->finalize = gst_compositor_finalize;
 
   gstelement_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_compositor_request_new_pad);
@@ -1107,6 +1594,36 @@ gst_compositor_class_init (GstCompositorClass * klass)
           GST_TYPE_COMPOSITOR_BACKGROUND,
           DEFAULT_BACKGROUND, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * compositor:zero-size-is-unscaled:
+   *
+   * Whether a pad with height or width 0 should be left unscaled
+   * in that dimension, or simply not composited in. Setting it to
+   * %FALSE might be useful when animating those properties.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_ZERO_SIZE_IS_UNSCALED,
+      g_param_spec_boolean ("zero-size-is-unscaled", "Zero size is unscaled",
+          "If TRUE, then input video is unscaled in that dimension "
+          "if width or height is 0 (for backwards compatibility)",
+          DEFAULT_ZERO_SIZE_IS_UNSCALED,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * compositor:max-threads:
+   *
+   * Maximum number of blending/rendering worker threads to spawn (0 = auto)
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_MAX_THREADS,
+      g_param_spec_uint ("max-threads", "Max Threads",
+          "Maximum number of blending/rendering worker threads to spawn "
+          "(0 = auto)", 0, G_MAXINT, DEFAULT_MAX_THREADS,
+          GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
+          G_PARAM_STATIC_STRINGS));
+
   gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
       &src_factory, GST_TYPE_AGGREGATOR_PAD);
   gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
@@ -1117,6 +1634,26 @@ gst_compositor_class_init (GstCompositorClass * klass)
       "Composite multiple video streams", "Wim Taymans <wim@fluendo.com>, "
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
+  /**
+   * compositor:ignore-inactive-pads:
+   *
+   * Don't wait for inactive pads when live. An inactive pad
+   * is a pad that hasn't yet received a buffer, but that has
+   * been waited on at least once.
+   *
+   * The purpose of this property is to avoid aggregating on
+   * timeout when new pads are requested in advance of receiving
+   * data flow, for example the user may decide to connect it later,
+   * but wants to configure it already.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class,
+      PROP_IGNORE_INACTIVE_PADS, g_param_spec_boolean ("ignore-inactive-pads",
+          "Ignore inactive pads",
+          "Avoid timing out waiting for inactive pads", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_PAD, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_OPERATOR, 0);
   gst_type_mark_as_plugin_api (GST_TYPE_COMPOSITOR_BACKGROUND, 0);
@@ -1127,6 +1664,8 @@ gst_compositor_init (GstCompositor * self)
 {
   /* initialize variables */
   self->background = DEFAULT_BACKGROUND;
+  self->zero_size_is_unscaled = DEFAULT_ZERO_SIZE_IS_UNSCALED;
+  self->max_threads = DEFAULT_MAX_THREADS;
 }
 
 /* GstChildProxy implementation */
@@ -1177,8 +1716,7 @@ plugin_init (GstPlugin * plugin)
 
   gst_compositor_init_blend ();
 
-  return gst_element_register (plugin, "compositor", GST_RANK_PRIMARY + 1,
-      GST_TYPE_COMPOSITOR);
+  return GST_ELEMENT_REGISTER (compositor, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
