@@ -1,6 +1,7 @@
 /* GStreamer encoding bin
  * Copyright (C) 2009 Edward Hervey <edward.hervey@collabora.co.uk>
  *           (C) 2009 Nokia Corporation
+ *           (C) 2020 Thibault Saunier <tsaunier@igalia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,21 +19,8 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GST_ENCODEBIN_H__
-#define __GST_ENCODEBIN_H__
+#pragma once
 
-#include <gst/gst.h>
-#include <gst/pbutils/pbutils.h>
+#include "gstencodebasebin.h"
 
-#define GST_TYPE_ENCODE_BIN               (gst_encode_bin_get_type())
-#define GST_ENCODE_BIN(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ENCODE_BIN,GstEncodeBin))
-#define GST_ENCODE_BIN_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ENCODE_BIN,GstEncodeBinClass))
-#define GST_IS_ENCODE_BIN(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ENCODE_BIN))
-#define GST_IS_ENCODE_BIN_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ENCODE_BIN))
-
-typedef struct _GstEncodeBin GstEncodeBin;
-typedef struct _GstEncodeBinClass GstEncodeBinClass;
-
-GType gst_encode_bin_get_type(void);
-
-#endif /* __GST_ENCODEBIN_H__ */
+G_DECLARE_FINAL_TYPE (GstEncodeBin, gst_encode_bin, GST, ENCODE_BIN, GstEncodeBaseBin);
\ No newline at end of file
