@@ -40,7 +40,7 @@
 #include <gst/gst-i18n-plugin.h>
 #include <string.h>             /* memset */
 
-#include "gsttcp.h"
+#include "gsttcpelements.h"
 #include "gsttcpserversink.h"
 
 #define TCP_BACKLOG             5
@@ -71,6 +71,8 @@ static void gst_tcp_server_sink_get_property (GObject * object, guint prop_id,
 #define gst_tcp_server_sink_parent_class parent_class
 G_DEFINE_TYPE (GstTCPServerSink, gst_tcp_server_sink,
     GST_TYPE_MULTI_SOCKET_SINK);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (tcpserversink, "tcpserversink",
+    GST_RANK_NONE, GST_TYPE_TCP_SERVER_SINK, tcp_element_init (plugin));
 
 static void
 gst_tcp_server_sink_class_init (GstTCPServerSinkClass * klass)
@@ -172,14 +174,20 @@ gst_tcp_server_sink_handle_server_read (GstTCPServerSink * sink)
     GInetSocketAddress *addr =
         G_INET_SOCKET_ADDRESS (g_socket_get_remote_address (client_socket,
             NULL));
-    gchar *ip =
-        g_inet_address_to_string (g_inet_socket_address_get_address (addr));
+    if (addr) {
+      gchar *ip =
+          g_inet_address_to_string (g_inet_socket_address_get_address (addr));
 
-    GST_DEBUG_OBJECT (sink, "added new client ip %s:%u with socket %p",
-        ip, g_inet_socket_address_get_port (addr), client_socket);
+      GST_DEBUG_OBJECT (sink, "added new client ip %s:%u with socket %p",
+          ip, g_inet_socket_address_get_port (addr), client_socket);
 
-    g_free (ip);
-    g_object_unref (addr);
+      g_free (ip);
+      g_object_unref (addr);
+    } else {
+      /* This can happen when the client immediately closes the connection */
+      GST_DEBUG_OBJECT (sink, "added new client (no address) with socket %p",
+          client_socket);
+    }
   }
 #endif
 
@@ -246,7 +254,7 @@ gst_tcp_server_sink_set_property (GObject * object, guint prop_id,
         break;
       }
       g_free (sink->host);
-      sink->host = g_strdup (g_value_get_string (value));
+      sink->host = g_value_dup_string (value);
       break;
     case PROP_PORT:
       sink->server_port = g_value_get_int (value);
@@ -288,43 +296,24 @@ gst_tcp_server_sink_init_send (GstMultiHandleSink * parent)
 {
   GstTCPServerSink *this = GST_TCP_SERVER_SINK (parent);
   GError *err = NULL;
-  GInetAddress *addr;
+  GList *addrs;
+  GList *cur_addr;
   GSocketAddress *saddr;
-  GResolver *resolver;
   gint bound_port;
 
-  /* look up name if we need to */
-  addr = g_inet_address_new_from_string (this->host);
-  if (!addr) {
-    GList *results;
+  addrs =
+      tcp_get_addresses (GST_ELEMENT (this), this->host,
+      this->element.cancellable, &err);
+  if (!addrs)
+    goto name_resolve;
 
-    resolver = g_resolver_get_default ();
-
-    results =
-        g_resolver_lookup_by_name (resolver, this->host,
-        this->element.cancellable, &err);
-    if (!results)
-      goto name_resolve;
-    addr = G_INET_ADDRESS (g_object_ref (results->data));
-
-    g_resolver_free_addresses (results);
-    g_object_unref (resolver);
-  }
-#ifndef GST_DISABLE_GST_DEBUG
-  {
-    gchar *ip = g_inet_address_to_string (addr);
-
-    GST_DEBUG_OBJECT (this, "IP address for host %s is %s", this->host, ip);
-    g_free (ip);
-  }
-#endif
-  saddr = g_inet_socket_address_new (addr, this->server_port);
-  g_object_unref (addr);
-
-  /* create the server listener socket */
+  /* iterate over addresses until one works */
+  cur_addr = addrs;
   this->server_socket =
-      g_socket_new (g_socket_address_get_family (saddr), G_SOCKET_TYPE_STREAM,
-      G_SOCKET_PROTOCOL_TCP, &err);
+      tcp_create_socket (GST_ELEMENT (this), &cur_addr, this->server_port,
+      &saddr, &err);
+  g_list_free_full (addrs, g_object_unref);
+
   if (!this->server_socket)
     goto no_socket;
 
@@ -379,19 +368,17 @@ no_socket:
     GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
         ("Failed to create socket: %s", err->message));
     g_clear_error (&err);
-    g_object_unref (saddr);
     return FALSE;
   }
 name_resolve:
   {
     if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-      GST_DEBUG_OBJECT (this, "Cancelled name resolval");
+      GST_DEBUG_OBJECT (this, "Cancelled name resolution");
     } else {
       GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
           ("Failed to resolve host '%s': %s", this->host, err->message));
     }
     g_clear_error (&err);
-    g_object_unref (resolver);
     return FALSE;
   }
 bind_failed:
