@@ -79,11 +79,13 @@ typedef struct _GstXImageSinkClass GstXImageSinkClass;
  * @heightmm: the height in millimeters of Display @disp
  * @par: the pixel aspect ratio calculated from @width, @widthmm and @height, 
  * @heightmm ratio
- * @use_xshm: used to known wether of not XShm extension is usable or not even
+ * @use_xshm: used to known whether of not XShm extension is usable or not even
+ * if the Extension is present
+ * @use_xkb: used to known wether of not Xkb extension is usable or not even
  * if the Extension is present
  * @caps: the #GstCaps that Display @disp can accept
  *
- * Structure used to store various informations collected/calculated for a
+ * Structure used to store various information collected/calculated for a
  * Display.
  */
 struct _GstXContext
@@ -107,6 +109,7 @@ struct _GstXContext
   GValue *par;                  /* calculated pixel aspect ratio */
 
   gboolean use_xshm;
+  gboolean use_xkb;
 
   GstCaps *caps;
   GstCaps *last_caps;
@@ -121,7 +124,7 @@ struct _GstXContext
  * through the #GstVideoOverlay interface
  * @gc: the Graphical Context of Window @win
  *
- * Structure used to store informations about a Window.
+ * Structure used to store information about a Window.
  */
 struct _GstXWindow
 {
@@ -152,7 +155,7 @@ struct _GstXWindow
  * @pool_lock: used to protect the buffer pool
  * @buffer_pool: a list of #GstXImageBuffer that could be reused at next buffer
  * allocation call
- * @synchronous: used to store if XSynchronous should be used or not (for 
+ * @synchronous: used to store if XSynchronous should be used or not (for
  * debugging purpose only)
  * @keep_aspect: used to remember if reverse negotiation scaling should respect
  * aspect ratio
@@ -205,6 +208,7 @@ struct _GstXImageSinkClass
 };
 
 GType gst_x_image_sink_get_type (void);
+GST_ELEMENT_REGISTER_DECLARE (ximagesink);
 
 G_END_DECLS
 #endif /* __GST_X_IMAGE_SINK_H__ */
