@@ -39,10 +39,18 @@
 
 #include <glib/gprintf.h>
 
+#ifdef HAVE_WINMM
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <timeapi.h>
+#endif
+
 #include "gst-play-kb.h"
 
 #define VOLUME_STEPS 20
 
+static gboolean wait_on_eos = FALSE;
+
 GST_DEBUG_CATEGORY (play_debug);
 #define GST_CAT_DEFAULT play_debug
 
@@ -53,7 +61,14 @@ typedef enum
   GST_PLAY_TRICK_MODE_DEFAULT_NO_AUDIO,
   GST_PLAY_TRICK_MODE_KEY_UNITS,
   GST_PLAY_TRICK_MODE_KEY_UNITS_NO_AUDIO,
-  GST_PLAY_TRICK_MODE_LAST
+  GST_PLAY_TRICK_MODE_LAST,
+
+  /* The instant-rate setting is a flag,
+   * applied on top of the trick-mode enum value.
+   * It needs to have a 2^n value bigger than 
+   * any of the enum values so setting it
+   * won't affect the trickmode value */
+  GST_PLAY_TRICK_MODE_INSTANT_RATE = (1 << 3)
 } GstPlayTrickMode;
 
 typedef enum
@@ -99,9 +114,14 @@ typedef struct
 
   GstPlayTrickMode trick_mode;
   gdouble rate;
+  gdouble start_position;
+
+  /* keyboard state tracking */
+  gboolean shift_pressed;
 } GstPlay;
 
 static gboolean quiet = FALSE;
+static gboolean instant_rate_changes = FALSE;
 
 static gboolean play_bus_msg (GstBus * bus, GstMessage * msg, gpointer data);
 static gboolean play_next (GstPlay * play);
@@ -137,17 +157,17 @@ gst_play_printf (const gchar * format, ...)
   va_end (args);
 
   if (len > 0 && str != NULL)
-    g_print ("%s", str);
+    gst_print ("%s", str);
 
   g_free (str);
 }
 
-#define g_print gst_play_printf
+#define gst_print gst_play_printf
 
 static GstPlay *
 play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
     gboolean gapless, gdouble initial_volume, gboolean verbose,
-    const gchar * flags_string, gboolean use_playbin3)
+    const gchar * flags_string, gboolean use_playbin3, gdouble start_position)
 {
   GstElement *sink, *playbin;
   GstPlay *play;
@@ -213,7 +233,7 @@ play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
     if (gst_value_deserialize (&val, flags_string))
       g_object_set_property (G_OBJECT (play->playbin), "flags", &val);
     else
-      g_printerr ("Couldn't convert '%s' to playbin flags!\n", flags_string);
+      gst_printerr ("Couldn't convert '%s' to playbin flags!\n", flags_string);
     g_value_unset (&val);
   }
 
@@ -247,7 +267,7 @@ play_new (gchar ** uris, const gchar * audio_sink, const gchar * video_sink,
 
   play->rate = 1.0;
   play->trick_mode = GST_PLAY_TRICK_MODE_NONE;
-
+  play->start_position = start_position;
   return play;
 }
 
@@ -305,8 +325,25 @@ play_set_relative_volume (GstPlay * play, gdouble volume_step)
   gst_stream_volume_set_volume (GST_STREAM_VOLUME (play->playbin),
       GST_STREAM_VOLUME_FORMAT_CUBIC, volume);
 
-  g_print (_("Volume: %.0f%%"), volume * 100);
-  g_print ("                  \n");
+  gst_print (_("Volume: %.0f%%"), volume * 100);
+  gst_print ("                  \n");
+}
+
+static void
+play_toggle_audio_mute (GstPlay * play)
+{
+  gboolean mute;
+
+  mute = gst_stream_volume_get_mute (GST_STREAM_VOLUME (play->playbin));
+
+  mute = !mute;
+  gst_stream_volume_set_mute (GST_STREAM_VOLUME (play->playbin), mute);
+
+  if (mute)
+    gst_print (_("Mute: on"));
+  else
+    gst_print (_("Mute: off"));
+  gst_print ("                  \n");
 }
 
 /* returns TRUE if something was installed and we should restart playback */
@@ -330,21 +367,26 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
       GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (play->playbin),
           GST_DEBUG_GRAPH_SHOW_ALL, "gst-play.async-done");
 
-      g_print ("Prerolled.\r");
+      gst_print ("Prerolled.\r");
       if (play->missing != NULL && play_install_missing_plugins (play)) {
-        g_print ("New plugins installed, trying again...\n");
+        gst_print ("New plugins installed, trying again...\n");
         --play->cur_idx;
         play_next (play);
       }
+      if (play->start_position > 0.0) {
+        play_do_seek (play, play->start_position * GST_SECOND,
+            play->rate, play->trick_mode);
+        play->start_position = 0;
+      }
       break;
     case GST_MESSAGE_BUFFERING:{
       gint percent;
 
       if (!play->buffering)
-        g_print ("\n");
+        gst_print ("\n");
 
       gst_message_parse_buffering (msg, &percent);
-      g_print ("%s %d%%  \r", _("Buffering..."), percent);
+      gst_print ("%s %d%%  \r", _("Buffering..."), percent);
 
       if (percent == 100) {
         /* a 100% message means buffering is done */
@@ -365,13 +407,13 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
       break;
     }
     case GST_MESSAGE_CLOCK_LOST:{
-      g_print (_("Clock lost, selecting a new one\n"));
+      gst_print (_("Clock lost, selecting a new one\n"));
       gst_element_set_state (play->playbin, GST_STATE_PAUSED);
       gst_element_set_state (play->playbin, GST_STATE_PLAYING);
       break;
     }
     case GST_MESSAGE_LATENCY:
-      g_print ("Redistribute latency...\n");
+      gst_print ("Redistribute latency...\n");
       gst_bin_recalculate_latency (GST_BIN (play->playbin));
       break;
     case GST_MESSAGE_REQUEST_STATE:{
@@ -382,7 +424,7 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
 
       gst_message_parse_request_state (msg, &state);
 
-      g_print ("Setting state to %s as requested by %s...\n",
+      gst_print ("Setting state to %s as requested by %s...\n",
           gst_element_state_get_name (state), name);
 
       gst_element_set_state (play->playbin, state);
@@ -392,10 +434,10 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
     case GST_MESSAGE_EOS:
       /* print final position at end */
       play_timeout (play);
-      g_print ("\n");
+      gst_print ("\n");
       /* and switch to next item in list */
-      if (!play_next (play)) {
-        g_print ("%s\n", _("Reached end of play list."));
+      if (!wait_on_eos && !play_next (play)) {
+        gst_print ("%s\n", _("Reached end of play list."));
         g_main_loop_quit (play->loop);
       }
       break;
@@ -408,9 +450,9 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
           GST_DEBUG_GRAPH_SHOW_ALL, "gst-play.warning");
 
       gst_message_parse_warning (msg, &err, &dbg);
-      g_printerr ("WARNING %s\n", err->message);
+      gst_printerr ("WARNING %s\n", err->message);
       if (dbg != NULL)
-        g_printerr ("WARNING debug information: %s\n", dbg);
+        gst_printerr ("WARNING debug information: %s\n", dbg);
       g_clear_error (&err);
       g_free (dbg);
       break;
@@ -424,9 +466,10 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
           GST_DEBUG_GRAPH_SHOW_ALL, "gst-play.error");
 
       gst_message_parse_error (msg, &err, &dbg);
-      g_printerr ("ERROR %s for %s\n", err->message, play->uris[play->cur_idx]);
+      gst_printerr ("ERROR %s for %s\n", err->message,
+          play->uris[play->cur_idx]);
       if (dbg != NULL)
-        g_printerr ("ERROR debug information: %s\n", dbg);
+        gst_printerr ("ERROR debug information: %s\n", dbg);
       g_clear_error (&err);
       g_free (dbg);
 
@@ -434,14 +477,14 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
       gst_element_set_state (play->playbin, GST_STATE_NULL);
 
       if (play->missing != NULL && play_install_missing_plugins (play)) {
-        g_print ("New plugins installed, trying again...\n");
+        gst_print ("New plugins installed, trying again...\n");
         --play->cur_idx;
         play_next (play);
         break;
       }
       /* try next item in list then */
       if (!play_next (play)) {
-        g_print ("%s\n", _("Reached end of play list."));
+        gst_print ("%s\n", _("Reached end of play list."));
         g_main_loop_quit (play->loop);
       }
       break;
@@ -470,15 +513,38 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
                   key = GST_PLAY_KB_ARROW_UP;
                 else if (strcmp (key, "Down") == 0)
                   key = GST_PLAY_KB_ARROW_DOWN;
-                else if (strcmp (key, "space") == 0)
-                  key = " ";
-                else if (strlen (key) > 1)
+                else if (strncmp (key, "Shift", 5) == 0) {
+                  play->shift_pressed = TRUE;
                   break;
+                } else if (strcmp (key, "space") == 0 ||
+                    strcmp (key, "Space") == 0) {
+                  key = " ";
+                } else if (strcmp (key, "minus") == 0) {
+                  key = "-";
+                } else if (strcmp (key, "plus") == 0
+                    /* TODO: That's not universally correct at all, but still handy */
+                    || (strcmp (key, "equal") == 0 && play->shift_pressed)) {
+                  key = "+";
+                } else if (strlen (key) > 1) {
+                  break;
+                }
 
                 keyboard_cb (key, user_data);
               }
               break;
             }
+            case GST_NAVIGATION_EVENT_KEY_RELEASE:
+            {
+              const gchar *key;
+
+              if (gst_navigation_event_parse_key_event (ev, &key)) {
+                GST_INFO ("Key release: %s", key);
+                if (strncmp (key, "Shift", 5) == 0) {
+                  play->shift_pressed = FALSE;
+                }
+              }
+              break;
+            }
             case GST_NAVIGATION_EVENT_MOUSE_BUTTON_PRESS:
             {
               gint button;
@@ -577,7 +643,7 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
             } else if (type & GST_STREAM_TYPE_TEXT) {
               play->cur_text_sid = g_strdup (stream_id);
             } else {
-              g_print ("Unknown stream type with stream-id %s", stream_id);
+              gst_print ("Unknown stream type with stream-id %s", stream_id);
             }
             gst_object_unref (stream);
           }
@@ -593,7 +659,7 @@ play_bus_msg (GstBus * bus, GstMessage * msg, gpointer user_data)
         gchar *desc;
 
         desc = gst_missing_plugin_message_get_description (msg);
-        g_print ("Missing plugin: %s\n", desc);
+        gst_print ("Missing plugin: %s\n", desc);
         g_free (desc);
         play->missing = g_list_append (play->missing, gst_message_ref (msg));
       }
@@ -634,7 +700,7 @@ play_timeout (gpointer user_data)
     pstr[9] = '\0';
     g_snprintf (dstr, 32, "%" GST_TIME_FORMAT, GST_TIME_ARGS (dur));
     dstr[9] = '\0';
-    g_print ("%s / %s %s\r", pstr, dstr, status);
+    gst_print ("%s / %s %s\r", pstr, dstr, status);
   }
 
   return TRUE;
@@ -666,7 +732,7 @@ play_uri (GstPlay * play, const gchar * next_uri)
   play_reset (play);
 
   loc = play_uri_get_display_name (play, next_uri);
-  g_print (_("Now playing %s\n"), loc);
+  gst_print (_("Now playing %s\n"), loc);
   g_free (loc);
 
   g_object_set (play->playbin, "uri", next_uri, NULL);
@@ -676,11 +742,11 @@ play_uri (GstPlay * play, const gchar * next_uri)
       /* ignore, we should get an error message posted on the bus */
       break;
     case GST_STATE_CHANGE_NO_PREROLL:
-      g_print ("Pipeline is live.\n");
+      gst_print ("Pipeline is live.\n");
       play->is_live = TRUE;
       break;
     case GST_STATE_CHANGE_ASYNC:
-      g_print ("Prerolling...\r");
+      gst_print ("Prerolling...\r");
       break;
     default:
       break;
@@ -729,8 +795,8 @@ play_about_to_finish (GstElement * playbin, gpointer user_data)
 
   next_uri = play->uris[next_idx];
   loc = play_uri_get_display_name (play, next_uri);
-  g_print (_("About to finish, preparing next title: %s"), loc);
-  g_print ("\n");
+  gst_print (_("About to finish, preparing next title: %s"), loc);
+  gst_print ("\n");
   g_free (loc);
 
   g_object_set (play->playbin, "uri", next_uri, NULL);
@@ -817,9 +883,9 @@ shuffle_uris (gchar ** uris, guint num)
   if (num < 2)
     return;
 
-  for (i = 0; i < num; i++) {
-    /* gets equally distributed random number in 0..num-1 [0;num[ */
-    j = g_random_int_range (0, num);
+  for (i = num - 1; i >= 1; i--) {
+    /* +1 because number returned will be in range [a;b[ so excl. stop */
+    j = g_random_int_range (0, i + 1);
     tmp = uris[j];
     uris[j] = uris[i];
     uris[i] = tmp;
@@ -843,7 +909,7 @@ toggle_paused (GstPlay * play)
   if (!play->buffering) {
     gst_element_set_state (play->playbin, play->desired_state);
   } else if (play->desired_state == GST_STATE_PLAYING) {
-    g_print ("\nWill play as soon as buffering finishes)\n");
+    gst_print ("\nWill play as soon as buffering finishes)\n");
   }
 }
 
@@ -878,7 +944,7 @@ relative_seek (GstPlay * play, gdouble percent)
   pos = pos + step;
   if (pos > dur) {
     if (!play_next (play)) {
-      g_print ("\n%s\n", _("Reached end of play list."));
+      gst_print ("\n%s\n", _("Reached end of play list."));
       g_main_loop_quit (play->loop);
     }
   } else {
@@ -892,7 +958,7 @@ relative_seek (GstPlay * play, gdouble percent)
 
 seek_failed:
   {
-    g_print ("\nCould not seek.\n");
+    gst_print ("\nCould not seek.\n");
   }
 }
 
@@ -930,7 +996,7 @@ play_do_seek (GstPlay * play, gint64 pos, gdouble rate, GstPlayTrickMode mode)
   if (!seekable)
     return FALSE;
 
-  seek_flags = GST_SEEK_FLAG_FLUSH;
+  seek_flags = 0;
 
   switch (mode) {
     case GST_PLAY_TRICK_MODE_DEFAULT:
@@ -951,6 +1017,19 @@ play_do_seek (GstPlay * play, gint64 pos, gdouble rate, GstPlayTrickMode mode)
       break;
   }
 
+  /* See if we can do an instant rate change (not changing dir) */
+  if (mode & GST_PLAY_TRICK_MODE_INSTANT_RATE && rate * play->rate > 0) {
+    seek = gst_event_new_seek (rate, GST_FORMAT_TIME,
+        seek_flags | GST_SEEK_FLAG_INSTANT_RATE_CHANGE,
+        GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE,
+        GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
+    if (gst_element_send_event (play->playbin, seek)) {
+      goto done;
+    }
+  }
+
+  /* No instant rate change, need to do a flushing seek */
+  seek_flags |= GST_SEEK_FLAG_FLUSH;
   if (rate >= 0)
     seek = gst_event_new_seek (rate, GST_FORMAT_TIME,
         seek_flags | GST_SEEK_FLAG_ACCURATE,
@@ -965,21 +1044,27 @@ play_do_seek (GstPlay * play, gint64 pos, gdouble rate, GstPlayTrickMode mode)
   if (!gst_element_send_event (play->playbin, seek))
     return FALSE;
 
+done:
   play->rate = rate;
-  play->trick_mode = mode;
+  play->trick_mode = mode & ~GST_PLAY_TRICK_MODE_INSTANT_RATE;
   return TRUE;
 }
 
 static void
 play_set_playback_rate (GstPlay * play, gdouble rate)
 {
-  if (play_set_rate_and_trick_mode (play, rate, play->trick_mode)) {
-    g_print (_("Playback rate: %.2f"), rate);
-    g_print ("                               \n");
+  GstPlayTrickMode mode = play->trick_mode;
+
+  if (instant_rate_changes)
+    mode |= GST_PLAY_TRICK_MODE_INSTANT_RATE;
+
+  if (play_set_rate_and_trick_mode (play, rate, mode)) {
+    gst_print (_("Playback rate: %.2f"), rate);
+    gst_print ("                               \n");
   } else {
-    g_print ("\n");
-    g_print (_("Could not change playback rate to %.2f"), rate);
-    g_print (".\n");
+    gst_print ("\n");
+    gst_print (_("Could not change playback rate to %.2f"), rate);
+    gst_print (".\n");
   }
 }
 
@@ -1027,9 +1112,10 @@ play_switch_trick_mode (GstPlay * play)
   mode_desc = trick_mode_get_description (new_mode);
 
   if (play_set_rate_and_trick_mode (play, play->rate, new_mode)) {
-    g_print ("Rate: %.2f (%s)                      \n", play->rate, mode_desc);
+    gst_print ("Rate: %.2f (%s)                      \n", play->rate,
+        mode_desc);
   } else {
-    g_print ("\nCould not change trick mode to %s.\n", mode_desc);
+    gst_print ("\nCould not change trick mode to %s.\n", mode_desc);
   }
 }
 
@@ -1087,7 +1173,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
   g_mutex_lock (&play->selection_lock);
   if (play->is_playbin3) {
     if (!play->collection) {
-      g_print ("No stream-collection\n");
+      gst_print ("No stream-collection\n");
       g_mutex_unlock (&play->selection_lock);
       return;
     }
@@ -1114,7 +1200,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
             cur_text_idx = nb_text;
           nb_text++;
         } else {
-          g_print ("Unknown stream type with stream-id %s", sid);
+          gst_print ("Unknown stream type with stream-id %s", sid);
         }
       }
     }
@@ -1200,7 +1286,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
   }
 
   if (n < 1) {
-    g_print ("No %s tracks.\n", name);
+    gst_print ("No %s tracks.\n", name);
     g_mutex_unlock (&play->selection_lock);
   } else {
     gchar *lcode = NULL, *lname = NULL;
@@ -1209,7 +1295,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
 
     if (cur >= n && track_type != GST_PLAY_TRACK_TYPE_VIDEO) {
       cur = -1;
-      g_print ("Disabling %s.           \n", name);
+      gst_print ("Disabling %s.           \n", name);
       if (play->is_playbin3) {
         /* Just make it empty for the track type */
       } else if (cur_flags & flag) {
@@ -1230,7 +1316,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
               (gchar *) gst_stream_get_stream_id (stream));
           tags = gst_stream_get_tags (stream);
         } else {
-          g_print ("Collection has no stream for track %d of %d.\n",
+          gst_print ("Collection has no stream for track %d of %d.\n",
               cur + 1, n);
         }
       } else {
@@ -1249,10 +1335,10 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
         gst_tag_list_unref (tags);
       }
       if (lang != NULL)
-        g_print ("Switching to %s track %d of %d (%s).\n", name, cur + 1, n,
+        gst_print ("Switching to %s track %d of %d (%s).\n", name, cur + 1, n,
             lang);
       else
-        g_print ("Switching to %s track %d of %d.\n", name, cur + 1, n);
+        gst_print ("Switching to %s track %d of %d.\n", name, cur + 1, n);
     }
     g_free (lcode);
     g_free (lname);
@@ -1263,7 +1349,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
         gst_element_send_event (play->playbin,
             gst_event_new_select_streams (selected_streams));
       else
-        g_print ("Can't disable all streams !\n");
+        gst_print ("Can't disable all streams !\n");
     } else {
       g_object_set (play->playbin, prop_cur, cur, NULL);
     }
@@ -1276,6 +1362,7 @@ play_cycle_track_selection (GstPlay * play, GstPlayTrackType track_type)
 static void
 print_keyboard_help (void)
 {
+  /* *INDENT-OFF* */
   static struct
   {
     const gchar *key_desc;
@@ -1290,6 +1377,7 @@ print_keyboard_help (void)
     "\342\206\220", N_("seek backward")}, {
     "\342\206\221", N_("volume up")}, {
     "\342\206\223", N_("volume down")}, {
+    "m", N_("toggle audio mute on/off")}, {
     "+", N_("increase playback rate")}, {
     "-", N_("decrease playback rate")}, {
     "d", N_("change playback direction")}, {
@@ -1299,9 +1387,10 @@ print_keyboard_help (void)
     "s", N_("change subtitle track")}, {
     "0", N_("seek to beginning")}, {
   "k", N_("show keyboard shortcuts")},};
+  /* *INDENT-ON* */
   guint i, chars_to_pad, desc_len, max_desc_len = 0;
 
-  g_print ("\n\n%s\n\n", _("Interactive mode - keyboard controls:"));
+  gst_print ("\n\n%s\n\n", _("Interactive mode - keyboard controls:"));
 
   for (i = 0; i < G_N_ELEMENTS (key_controls); ++i) {
     desc_len = g_utf8_strlen (key_controls[i].key_desc, -1);
@@ -1311,11 +1400,11 @@ print_keyboard_help (void)
 
   for (i = 0; i < G_N_ELEMENTS (key_controls); ++i) {
     chars_to_pad = max_desc_len - g_utf8_strlen (key_controls[i].key_desc, -1);
-    g_print ("\t%s", key_controls[i].key_desc);
-    g_print ("%-*s: ", chars_to_pad, "");
-    g_print ("%s\n", key_controls[i].key_help);
+    gst_print ("\t%s", key_controls[i].key_desc);
+    gst_print ("%-*s: ", chars_to_pad, "");
+    gst_print ("%s\n", key_controls[i].key_help);
   }
-  g_print ("\n");
+  gst_print ("\n");
 }
 
 static void
@@ -1342,7 +1431,7 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
     case 'n':
     case '>':
       if (!play_next (play)) {
-        g_print ("\n%s\n", _("Reached end of play list."));
+        gst_print ("\n%s\n", _("Reached end of play list."));
         g_main_loop_quit (play->loop);
       }
       break;
@@ -1393,6 +1482,9 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
     case '0':
       play_do_seek (play, 0, play->rate, play->trick_mode);
       break;
+    case 'm':
+      play_toggle_audio_mute (play);
+      break;
     default:
       if (strcmp (key_input, GST_PLAY_KB_ARROW_RIGHT) == 0) {
         relative_seek (play, +0.08);
@@ -1411,6 +1503,43 @@ keyboard_cb (const gchar * key_input, gpointer user_data)
   }
 }
 
+#ifdef HAVE_WINMM
+static guint
+enable_winmm_timer_resolution (void)
+{
+  TIMECAPS time_caps;
+  guint resolution = 0;
+  MMRESULT res;
+
+  res = timeGetDevCaps (&time_caps, sizeof (TIMECAPS));
+  if (res != TIMERR_NOERROR) {
+    g_warning ("timeGetDevCaps() returned non-zero code %d", res);
+    return 0;
+  }
+
+  resolution = MIN (MAX (time_caps.wPeriodMin, 1), time_caps.wPeriodMax);
+  res = timeBeginPeriod (resolution);
+  if (res != TIMERR_NOERROR) {
+    g_warning ("timeBeginPeriod() returned non-zero code %d", res);
+    return 0;
+  }
+
+  gst_println (_("Use Windows high-resolution clock, precision: %u ms\n"),
+      resolution);
+
+  return resolution;
+}
+
+static void
+clear_winmm_timer_resolution (guint resolution)
+{
+  if (resolution == 0)
+    return;
+
+  timeEndPeriod (resolution);
+}
+#endif
+
 int
 main (int argc, char **argv)
 {
@@ -1422,6 +1551,7 @@ main (int argc, char **argv)
   gboolean gapless = FALSE;
   gboolean shuffle = FALSE;
   gdouble volume = -1;
+  gdouble start_position = 0;
   gchar **filenames = NULL;
   gchar *audio_sink = NULL;
   gchar *video_sink = NULL;
@@ -1432,6 +1562,9 @@ main (int argc, char **argv)
   GOptionContext *ctx;
   gchar *playlist_file = NULL;
   gboolean use_playbin3 = FALSE;
+#ifdef HAVE_WINMM
+  guint winmm_timer_resolution = 0;
+#endif
   GOptionEntry options[] = {
     {"verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
         N_("Output status information and property notifications"), NULL},
@@ -1453,14 +1586,25 @@ main (int argc, char **argv)
         N_("Disable interactive control via the keyboard"), NULL},
     {"volume", 0, 0, G_OPTION_ARG_DOUBLE, &volume,
         N_("Volume"), NULL},
+    {"start-position", 's', 0, G_OPTION_ARG_DOUBLE, &start_position,
+        N_("Start position in seconds."), NULL},
     {"playlist", 0, 0, G_OPTION_ARG_FILENAME, &playlist_file,
         N_("Playlist file containing input media files"), NULL},
+    {"instant-rate-changes", 'i', 0, G_OPTION_ARG_NONE, &instant_rate_changes,
+          N_
+          ("Use the experimental instant-rate-change flag when changing rate"),
+        NULL},
     {"quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet,
         N_("Do not print any output (apart from errors)"), NULL},
     {"use-playbin3", 0, 0, G_OPTION_ARG_NONE, &use_playbin3,
           N_("Use playbin3 pipeline")
           N_("(default varies depending on 'USE_PLAYBIN' env variable)"),
         NULL},
+    {"wait-on-eos", 0, 0, G_OPTION_ARG_NONE, &wait_on_eos,
+          N_
+          ("Keep showing the last frame on EOS until quit or playlist change command "
+              "(gapless is ignored)"),
+        NULL},
     {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, NULL},
     {NULL}
   };
@@ -1481,7 +1625,7 @@ main (int argc, char **argv)
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
-    g_print ("Error initializing: %s\n", GST_STR_NULL (err->message));
+    gst_print ("Error initializing: %s\n", GST_STR_NULL (err->message));
     g_option_context_free (ctx);
     g_clear_error (&err);
     return 1;
@@ -1494,9 +1638,9 @@ main (int argc, char **argv)
     gchar *version_str;
 
     version_str = gst_version_string ();
-    g_print ("%s version %s\n", g_get_prgname (), PACKAGE_VERSION);
-    g_print ("%s\n", version_str);
-    g_print ("%s\n", GST_PACKAGE_ORIGIN);
+    gst_print ("%s version %s\n", g_get_prgname (), PACKAGE_VERSION);
+    gst_print ("%s\n", version_str);
+    gst_print ("%s\n", GST_PACKAGE_ORIGIN);
     g_free (version_str);
 
     g_free (audio_sink);
@@ -1506,6 +1650,9 @@ main (int argc, char **argv)
     return 0;
   }
 
+  if (wait_on_eos)
+    gapless = FALSE;
+
   playlist = g_ptr_array_new ();
 
   if (playlist_file != NULL) {
@@ -1525,7 +1672,7 @@ main (int argc, char **argv)
       g_strfreev (lines);
       g_free (playlist_contents);
     } else {
-      g_printerr ("Could not read playlist: %s\n", err->message);
+      gst_printerr ("Could not read playlist: %s\n", err->message);
       g_clear_error (&err);
     }
     g_free (playlist_file);
@@ -1533,10 +1680,10 @@ main (int argc, char **argv)
   }
 
   if (playlist->len == 0 && (filenames == NULL || *filenames == NULL)) {
-    g_printerr (_("Usage: %s FILE1|URI1 [FILE2|URI2] [FILE3|URI3] ..."),
+    gst_printerr (_("Usage: %s FILE1|URI1 [FILE2|URI2] [FILE3|URI3] ..."),
         "gst-play-" GST_API_VERSION);
-    g_printerr ("\n\n"),
-        g_printerr ("%s\n\n",
+    gst_printerr ("\n\n"),
+        gst_printerr ("%s\n\n",
         _("You must provide at least one filename or URI to play."));
     /* No input provided. Free array */
     g_ptr_array_free (playlist, TRUE);
@@ -1567,33 +1714,54 @@ main (int argc, char **argv)
 
   /* prepare */
   play = play_new (uris, audio_sink, video_sink, gapless, volume, verbose,
-      flags, use_playbin3);
+      flags, use_playbin3, start_position);
 
   if (play == NULL) {
-    g_printerr
+    gst_printerr
         ("Failed to create 'playbin' element. Check your GStreamer installation.\n");
     return EXIT_FAILURE;
   }
+#ifdef HAVE_WINMM
+  /* Enable high-precision clock which will improve accuracy of various
+   * Windows timer APIs (e.g., Sleep()), and it will increase the precision
+   * of GstSystemClock as well
+   */
+
+  /* NOTE: Once timer resolution is updated via timeBeginPeriod(),
+   * application should undo it by calling timeEndPeriod()
+   *
+   * Prior to Windows 10, version 2004, timeBeginPeriod() affects global
+   * Windows setting (meaning that it will affect other processes),
+   * but starting with Windows 10, version 2004, this function no longer
+   * affects global timer resolution
+   */
+  winmm_timer_resolution = enable_winmm_timer_resolution ();
+#endif
 
   if (interactive) {
     if (gst_play_kb_set_key_handler (keyboard_cb, play)) {
-      g_print (_("Press 'k' to see a list of keyboard shortcuts.\n"));
+      gst_print (_("Press 'k' to see a list of keyboard shortcuts.\n"));
       atexit (restore_terminal);
     } else {
-      g_print ("Interactive keyboard handling in terminal not available.\n");
+      gst_print ("Interactive keyboard handling in terminal not available.\n");
     }
   }
 
   /* play */
   do_play (play);
 
+#ifdef HAVE_WINMM
+  /* Undo timeBeginPeriod() if required */
+  clear_winmm_timer_resolution (winmm_timer_resolution);
+#endif
+
   /* clean up */
   play_free (play);
 
   g_free (audio_sink);
   g_free (video_sink);
 
-  g_print ("\n");
+  gst_print ("\n");
   gst_deinit ();
   return 0;
 }
